<?php

/**
 * @file
 * Administrative page callbacks for codes module.
 */

function codes_update_code($code) {
  // Make sure tags, entities and attributes are well-formed and properly nested.
  $code['description'] = _filter_htmlcorrector($code['description']);

  db_query("UPDATE {code} SET title = '%s', module = '%s', description = '%s' WHERE name = '%s'", $code['title'], $code['module'], $code['description'], $code['name']);
}

/**
 * Code admin page.
 */
function codes_overview() {
  $header = array(
    array('data' => t('Code name'), 'field' => 'c.name', 'sort' => 'asc'),
    array('data' => t('Title'), 'field' => 'c.title'),
    array('data' => t('Module'), 'field' => 'c.module'),
    array('data' => t('Operations'), 'colspan' => 2),
  );

  $form['header'] = array(
   '#type' => 'value',
   '#value' => $header,
  );

  $sql = 'SELECT * FROM {code} c';
  $sql .= tablesort_sql($header);

  $result = pager_query($sql, 50, 0);

  $form['code']['#tree'] = TRUE;

  while ($code = db_fetch_object($result)) {
    $form['code'][$code->coid] = codes_row($code);
  }

  $form['pager'] = array('#value' => theme('pager', NULL, 50, 0));
  $form['#theme'] = 'codes_overview';

  return $form;
}

/**
 * One row of code table.
 */
function codes_row($code) {
  $form['name'] = array(
    '#type' => 'markup',
    '#value' => $code->name,
  );
  $form['title'] = array(
    '#type' => 'markup',
    '#value' => $code->title,
  );
  $form['module'] = array(
    '#type' => 'markup',
    '#value' => $code->module,
  );
  $edit_code_link = (0) ? '' : l(t('Edit code'), 'admin/build/codes/edit/'. $code->name);
  $form['edit_code'] = array(
    '#type' => 'markup',
    '#value' => $edit_code_link,
  );
  $edit_options_link = (0) ? '' : l(t('Edit options'), 'admin/build/codes-options/'. $code->name);
  $form['edit_options'] = array(
    '#type' => 'markup',
    '#value' => $edit_options_link,
  );

  return $form;
}

/**
 * Theme the code overview form.
 *
 * @ingroup themeable
 */
function theme_codes_overview($form) {
  $header = $form['header']['#value'];
  $rows = array();

  foreach (element_children($form['code']) as $key) {
     // Build the table row.
    $rows[] = array(
      array('data' => drupal_render($form['code'][$key]['name'])),
      array('data' => drupal_render($form['code'][$key]['title'])),
      array('data' => drupal_render($form['code'][$key]['module'])),
      array('data' => drupal_render($form['code'][$key]['edit_code'])),
      array('data' => drupal_render($form['code'][$key]['edit_options'])),
    );
  }

  $output = theme('table', $header, $rows);
  if ($form['pager']['#value']) {
    $output .= drupal_render($form['pager']);
  }
  $output .= drupal_render($form);

  return $output;
}

/**
 * Codes module settings.
 */
function codes_configure() {
  $form['codes_missing_text'] = array(
    '#type' => 'textfield',
    '#title' => t('Default string for missing option'),
    '#default_value' => variable_get('codes_missing_text', t('unknown')),
    '#description' => t('This text will be displayed when the code otion is not found. Use <em>&lt;code option&gt;</em> to display the code option.'),
    '#required' => TRUE,
  );
  $form['codes_show_code'] = array(
    '#type' => 'checkbox',
    '#title' => t('Show internal code'),
    '#default_value' => variable_get('codes_show_code', FALSE),
    '#description' => t('Show code option instead of display value. Used for development.'),
    '#required' => TRUE,
  );
  return system_settings_form($form);
}

/**
 * Edit code.
 */
function codes_edit(&$form_state, $type, $code) {
  if ($type == 'edit') {
    $form['name'] = array(
      '#type' => 'value',
      '#value' => $code['name'],
    );
    $form['#insert'] = FALSE;
    $form['delete'] = array(
      '#type' => 'submit',
      '#value' => t('Delete'),
      '#access' => TRUE,
      '#submit' => array('codes_delete_submit'),
      '#weight' => 10,
    );
  }
  else {
    $code = array(
      'name' => '',
      'title' => '',
      'module' => '',
      'description' => '',
    );
    $form['name'] = array(
      '#type' => 'textfield',
      '#title' => t('Code name'),
      '#maxsize' => 255,
      '#description' => t('The machine-readable name of this code. This text will be used as a reference. This name must contain only lowercase letters, numbers and hyphens, and must be unique.'),
      '#required' => TRUE,
    );
    $form['#insert'] = TRUE;
  }
  $form['#title'] = $code['title'];
  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#default_value' => $code['title'],
    '#description' => t('Will be used as title for dropdowns or radio button groups.'),
    '#required' => TRUE,
  );
  $form['module'] = array(
    '#type' => 'textfield',
    '#title' => t('Module'),
    '#default_value' => $code['module'],
    '#description' => t('The owner module, where this code is used.'),
    '#required' => TRUE,
  );
  $form['description'] = array(
    '#type' => 'textarea',
    '#title' => t('Description'),
    '#default_value' => $code['description'],
    '#description' => t('The short description.'),
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );

  return $form;
}

/**
 * Validates the human and machine-readable names when adding or editing a code.
 */
function codes_edit_validate($form, &$form_state) {
  if ($form['#insert']) {
    $code_name = $form_state['values']['name'];
    $code_exists = db_result(db_query("SELECT COUNT(*) FROM {code} WHERE name = '%s'", $code_name));
    if ($code_exists) {
      form_set_error('codes_edit', t('Code %code_name already exists.', array('%code_name' => $code_name)));
    }
  }
}

/**
 * Submit function for adding or editing a code (Codes).
 */
function codes_edit_submit($form, &$form_state) {
  $code = $form_state['values'];
  $path = 'admin/build/codes-options/';
  if ($form['#insert']) {
    codes_create_code($code);
  }
  else {
    codes_update_code($code);
  }
  $form_state['redirect'] = $path . $code['name'];
}

/**
 * Menu callback; Check access and present a confirm form for deleting a code.
 */
function codes_delete_code($code) {
  $protected_option_exists = db_result(db_query("SELECT COUNT(*) FROM {code_option} WHERE coid = %d AND protected = 1", $code['coid']));
  if ($protected_option_exists) {
    drupal_set_message(t('There are protected options associated to code %code_name. You must delete those code options first.', array('%code_name' => $code['name'])), 'error');
    drupal_goto('admin/build/codes');
    return;
  }
  return drupal_get_form('codes_delete_code_confirm', $code);
}

/**
 * Build a confirm form for deletion of code.
 */
function codes_delete_code_confirm(&$form_state, $code) {
  $form['#code'] = $code;
  return confirm_form($form, t("Are you sure you want to delete the code %code_name and all the options associated to it?", array('%code_name' => $code['name'])), 'admin/build/codes', NULL, t('Delete'));
}

/**
 * Delete a code.
 */
function codes_delete_code_confirm_submit($form, &$form_state) {
  $form_state['redirect'] = 'admin/build/codes';
  db_query("DELETE FROM {code_option} WHERE coid = %d", $form['#code']['coid']);
  db_query("DELETE FROM {code} WHERE coid = %d", $form['#code']['coid']);
  drupal_set_message(t('The code %code_name has been deleted.', array('%code_name' => $form['#code']['name'])));
}

/**
 * Submit function for the 'Delete' button on the code editing form.
 */
function codes_delete_submit($form, &$form_state) {
  $form_state['redirect'] = 'admin/build/codes/delete/'. $form_state['values']['name'];
}
