<?php

/**
 * @file
 * Codes functions.
 */

function codes_retrieve($code_name, $form_type, $subset_name = 'selectable') {
  $item = array();
  $code_name = check_plain($code_name);

  $code = code_load($code_name);
  if (empty($code['coid'])) {
    drupal_set_message(t('Missing code %code_name.', array('%code_name' => $code_name)), 'error');
    $item['#type'] = 'value';
    $item['#value'] = $code_name;
    return $item;
  }

  $result = db_query("SELECT * FROM {code_option} WHERE coid = '%s'", $code['coid']);

  $item['#type'] = $form_type;
  $item['#title'] = $code['title'];
  $item['#description'] = $code['description'];
  switch ($form_type) {
    case 'full': // Special case, return every property.
      $item['#default_multi'] = $code['default_multi'];
      $item['#default_single'] = $code['default_single'];
      break;
    case 'checkboxes':
      $item['#default_value'] = $code['default_multi'];
      break;
    default:
      $item['#default_value'] = $code['default_single'];
  }

  $filter = FALSE;

  if (module_exists('codes_subset')) {
    $filter = (in_array($subset_name, $code['subset']));
  }

  while ($option = db_fetch_array($result)) {
    if (!$filter || !in_array($option['name'], $code['subset'][$subset_name])) {
      $item['#options'][$option['name']] = $option['title'];
      if ($form_type = 'full') {// Special case, return every property.
        $item['#options'][$option['name']]['name'] = $option['name'];
        $item['#options'][$option['name']]['title'] = $option['title'];
        $item['#options'][$option['name']]['description'] = $option['description'];
      }
    }
  }
  return $item;
}

function codes_create_code_option($coid, $option) {
  if (!isset($option['descripton'])) {
    $option['descrtiption'] = '';
  }
  if (!isset($option['protected'])) {
    $option['protected'] = 1;
  }
  if (!isset($option['weight'])) {
    $option['weight'] = 99;
  }
  db_query("INSERT INTO {code_option} (coid, name, title, description, protected, weight) VALUES (%d, '%s', '%s', '%s', %d, %d)", $coid, $option['name'], $option['title'], $option['description'], $option['protected'], $option['weight']);

}

function codes_create_code($code) {
  $code_exists = db_result(db_query("SELECT COUNT(*) FROM {code} WHERE name = '%s'", $code['name']));
  if (!$code_exists) {
    if (!isset($code['descripton'])) {
      $code['description'] = '';
    }
    if (!isset($code['default_single'])) {
      $code['default_single'] = '';
    }
    if (!isset($code['default_multi'])) {
      $code['default_multi'] = array();
    }

    // Make sure tags, entities and attributes are well-formed and properly nested.
    $code['description'] = _filter_htmlcorrector($code['description']);

    db_query("INSERT INTO {code} (name, title, module, description, default_single, default_multi) VALUES ('%s', '%s', '%s', '%s', '%s', '%s')", $code['name'], $code['title'], $code['module'], $code['description'], $code['default_single'], serialize($code['default_multi']));
    drupal_set_message(t('Code %code_name created.', array('%code_name' => $code['name'])));
  }

  if (!isset($code['options'])) {
    return;
  }

  $coid = db_result(db_query("SELECT coid FROM {code} WHERE name = '%s'", $code['name']));
  foreach ($code['options'] as $option_name => $option) {
    $option['name'] = $option_name;
    codes_create_code_option($coid, $option);
  }

  if ($code_exists) {
    drupal_set_message(t('Code %code_name updated.', array('%code_name' => $code['name'])));
  }
}

function drupal_install_codes($module) {
  $codes = drupal_get_codes_unprocessed($module);
  _drupal_initialize_codes($module, $codes);

  foreach ($codes as $code) {
    codes_create_code($code);
  }
}


function drupal_uninstall_codes($module) {
  $codes = drupal_get_codes_unprocessed($module);
  _drupal_initialize_codes($module, $codes);

  foreach ($codes as $code) {
    //drupal_delete_code($code['name']);
  }
}


function drupal_get_codes_unprocessed($module, $code = NULL) {
  // Load the .install file to get hook_codes.
  module_load_include('install', $module);
  $codes = module_invoke($module, 'codes');

  if (!is_null($codes) && isset($codes[$code])) {
    return $codes[$code];
  }
  else {
    return $codes;
  }
}

/**
 * Fill in required default values for code definitions returned by hook_codes().
 *
 * @param $module
 *   The module for which hook_codes() was invoked.
 * @param $codes
 *   The codes definition array as it was returned by the module's
 *   hook_codes().
 */
function _drupal_initialize_codes($module, &$codes) {
  // Set the name and module key for all codes.
  foreach ($codes as $name => $code) {
    if (empty($code['module'])) {
      $codes[$name]['module'] = $module;
    }
    if (!isset($code['name'])) {
      $codes[$name]['name'] = $name;
    }
  }
}
