<?php

/**
 * @file
 * Administrative page callbacks for codes module (Options).
 */

/**
 * Code options admin page.
 */
function codes_overview_options(&$form_state, $code) {
  $result = db_query("SELECT * FROM {code_option} WHERE coid = '%d' ORDER BY weight", $code['coid']);

  $header = array(
    array('data' => t('Name')),
    array('data' => t('Title')),
    array('data' => t('Weight')),
    //t('Selectable'),
    array('data' => t('Default (single/multi)'), 'colspan' => 2),
    array('data' => t('Protected')),
    array('data' => t('Operations')),
  );

  if (module_exists('codes_subset')) {
    $subsets_definition = codes_retrieve('c-subsets', 'full');
    $subsets = $code['subset'];
  }

  $form['header'] = array(
   '#type' => 'value',
   '#value' => $header,
  );

  $form['option']['#tree'] = TRUE;
  $form['#cache'] = TRUE;
  $form['#code'] = $code;

  $default_single_options = array();
  $default_single = -1;
  $form['option'][0] = codes_option_new_row();

  while ($option = db_fetch_object($result)) {
    $option->code_name = $code['name'];
    $form['option'][$option->opid] = codes_option_row($option);
    //$form['option'][$option->opid] = codes_option_row_subset($option);

    $form['option'][$option->opid]['default_multi'] = array(
    '#type' => 'checkbox',
    '#default_value' => (in_array($option->name, $code['default_multi'])),
  );
    if ($option->name === $code['default_single']) {
      $default_single = $option->opid;
    }
    $default_single_options[$option->opid] = '';
  }

  $form['default_single'] = array(
    '#type' => 'radios',
    '#options' => $default_single_options,
    '#default_value' => $default_single,
  );

  $form['#theme'] = 'codes_overview_options';

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );

  return $form;
}

/**
 * Addiditonal subsets in row of code option table.
 */
function codes_option_row_subset($option) {
  $form['default_multi'] = array(
    '#type' => 'checkbox',
    '#default_value' => $option->selectable,
  );
  return $form;
}

/**
 * One row of code option table.
 */
function codes_option_row($option) {
  $access_protected = user_access('administer protected code');
  $protected = $option->protected && !$access_protected;

  $form['name'] = array(
    '#type' => 'textfield',
    '#default_value' => $option->name,
    '#size' => 20,
    '#maxlength' => 255,
    '#attributes' => ($protected) ? array('disabled' => 'disabled') : array(),
  );
  $form['title'] = array(
    '#type' => 'textfield',
    '#default_value' => $option->title,
    '#size' => 35,
    '#maxlength' => 255,
  );
  $form['weight'] = array(
    '#type' => 'weight',
    '#default_value' => $option->weight,
    '#delta' => 100,
  );
  $form['protected'] = array(
    '#type' => 'checkbox',
    '#default_value' => $option->protected,
    '#attributes' => (!$access_protected) ? array('disabled' => 'disabled') : array(),
  );
  $delete_link = ($option->opid <= 0) ? '' : l(t('Delete'), 'admin/build/codes-options/'. $option->code_name .'/delete/'. $option->name);
  $form['delete'] = array(
    '#type' => 'markup',
    '#value' => $delete_link,
  );

  return $form;
}

/**
 * New row of code option table.
 * @note: not used yet
 */
function codes_option_new_row() {
  $option_new->name = '';
  $option_new->title = '';
  $option_new->weight = -100;
  //$option_new->selectable = 1;
  $option_new->protected = 0;
  $option_new->opid = 0;
  return codes_option_row($option_new);
}

/**
 * Theme the code options overview form.
 *
 * @ingroup themeable
 */
function theme_codes_overview_options($form) {
  $header = $form['header']['#value'];
  $rows = array();

  foreach (element_children($form['option']) as $key) {
    $form['option'][$key]['weight']['#attributes']['class'] = 'option-order-weight';
    // Build the table row.
    $rows[] = array(
      'data' => array(
        drupal_render($form['option'][$key]['name']),
        drupal_render($form['option'][$key]['title']),
        drupal_render($form['option'][$key]['weight']),
        drupal_render($form['default_single'][$key]),
        drupal_render($form['option'][$key]['default_multi']),
        drupal_render($form['option'][$key]['protected']),
        drupal_render($form['option'][$key]['delete'])
      ),
      'class' => 'draggable',
    );
  }

  $output = $form['#code']['description'];
  $output .= theme('table', $header, $rows, array('id' => 'codes-options-table'));
  $output .= drupal_render($form);

  drupal_add_tabledrag('codes-options-table', 'order', 'sibling', 'option-order-weight');

  return $output;
}


/**
 * Submit handler for code options admin page.
 */
function codes_overview_options_submit($form, &$form_state) {
  $code = $form['#code'];
  $path = 'admin/build/codes-options/';
  $default_single = '';
  $default_multi = array();

  foreach ($form_state['values']['option'] as $opid => $option) {
    if ($opid <= 0) { // This is a new option, insert required.
      if (!empty($option['name'])) {
        // Check to see whether the code option already exists.
        $num_rows = db_result(db_query("SELECT COUNT(*) FROM {code_option} WHERE coid = %d AND name = '%s'", $code['coid'], $option['name']));
        if ($num_rows) {
          drupal_set_message(t('Code option %option_name already exists.', array('%option_name' => $option['name'])), 'error');
        }
        else {
          db_query("INSERT INTO {code_option} (coid, name, title, weight, protected) VALUES ('%d', '%s', '%s', %d, %d)", $code['coid'], $option['name'], $option['title'], $option['weight'], $option['protected']);
        }
      }
    }
    else { //This is an existing option, update required.
      // @todo Check to see whether the record needs updating.
      db_query("UPDATE {code_option} SET name = '%s', title = '%s', weight = %d, protected = %d WHERE opid = %d", $option['name'], $option['title'], $option['weight'], $option['protected'], $opid);
      if ($option['default_multi']) {
        // Store the unchenked options, that should be less.
        array_push($default_multi, $option['name']);
      }

      if ($opid == $form_state['values']['default_single']) {
        $default_single = $option['name'];
      }
    }
  }
  db_query("UPDATE {code} SET default_single = '%s', default_multi = '%s' WHERE coid = %d", $default_single, serialize($default_multi), $code['coid']);

  $form_state['redirect'] = $path . $code['name'];
}


/**
 * Menu callback; Check access and present a confirm form for deleting a code option.
 */
function codes_delete_option($code, $option_name) {
  $option = db_fetch_array(db_query("SELECT * FROM {code_option} WHERE coid = %d AND name = '%s'", $code['coid'], $option_name));

  // Deletion of protected code option needs proper permission.
  if ($option['protected'] && !user_access('administer protected code')) {
    drupal_access_denied();
    return;
  }
  return drupal_get_form('codes_delete_option_confirm', $code, $option);
}


/**
 * Build a confirm form for deletion of code option.
 */
function codes_delete_option_confirm(&$form_state, $code, $option) {
  $form['#code'] = $code;
  $form['#option'] = $option;
  return confirm_form($form, t("Are you sure you want to delete the code option %option_name from code %code_name?", array('%option_name' => $option['name'], '%code_name' => $code['name'])), 'admin/build/codes-options/'. $code['name'], NULL, t('Delete'));
}


/**
 * Delete a code option.
 */
function codes_delete_option_confirm_submit($form, &$form_state) {
  $form_state['redirect'] = 'admin/build/codes-options/'. $form['#code']['name'];
  db_query("DELETE FROM {code_option} WHERE opid = %d", $form['#option']['opid']);
  drupal_set_message(t('The code option %option_name has been deleted.', array('%option_name' => $form['#option']['name'])));
}

